// LERA: Tune Mixer
unit lrmixr;

interface
uses Inferno, Mikky, Alf, Ini, lrwvfm, X11, XLib, XUtil, X, dos, VGAColors;

//   
procedure mixrChangeVolume(const cL, cR: ShortInt);

//     mixr??Cf  wv?freq
procedure mixrNewFile;

const
  mixrWnd		: TWindow = 0;	//    ,   
					//     ...
  mixrNewUserFreq	: Boolean = False; //     

var
  mixrDACf, mixrADCf	: Word;		//    OSS


implementation
type
  tVolDesc	= Record
    vp		: ^ShortInt;
    s		: PChar;
  end;

  tFreqDesc	= Record
    fp		: ^Word;
    s		: PChar
  end;

const
  VolL		: ShortInt = 50;
  VolR		: ShortInt = 50;
  MaxVol	: LongInt = 100;
  MixerCmd	: String = '/usr/sbin/mixer rec %l:%r';
  VolLdesc	: tVolDesc = (vp: @VolL; s: '');
  VolRdesc	: tVolDesc = (vp: @VolR; s: '');
  FreqIdesc	: tFreqDesc = (fp: @mixrADCf; s: '');
  FreqOdesc	: tFreqDesc = (fp: @mixrDACf; s: '');
  FontLabel	: String = '-*-fixed-medium-r-*-*-*-*-*-*-*-*-koi8-*';

  Border	= 8;
  WndX		: LongInt = 000;
  WndY		: LongInt = 237;
  WndW		: LongInt = 272;
  WndH		: LongInt = 114;

  Remark	= '    \  DAC  ADC.';

var
  OldFullingTime	: TFullingTime;
  OldExitProc		: Pointer;
  msL, msR, msI, msO	: RngMouseArea;
  LabH,
  FreqW, FreqH,
  LevelW, LevelH	: LongInt;
  Colors		: TColors;


procedure mixrNewFile;
begin mixrDACf := wvRfreq; mixrADCf := wvWfreq; DrawButton(msI); DrawButton(msO) end;


procedure mixrChangeVolume(const cL, cR: ShortInt);
const
  prevL: ShortInt = -1;
  prevR: ShortInt = -1;

var rc: String;

  procedure regex(const c: Char; var v: ShortInt; const d: ShortInt);
  var p: LongInt;
  begin
    inc(v, d);
    if v < 0 then v := 0;
    if v > MaxVol then v := MaxVol;
    p := pos('%'+c, rc);
    if p <> 0 then begin
      delete(rc, p, 2);
      insert(ValToStrG(v), rc, p);
    end;
  end;

begin if MixerCMD <> '' then begin
  rc := MixerCMD;
  regex('l', VolL, cL); regex('r', VolR, cR);
  if (prevL <> VolL) or (prevR <> VolR) then begin
    Exec(rc, '');
    if Lo(DosExitCode) <> 0 then AddLog('Cannot exec "' + rc + '". DosExitCode = $' + HexStr(DosExitCode, 4));
    DrawButton(msL); DrawButton(msR);
    prevL := VolL; prevR := VolR;
  end;
end end;


//  
procedure intSizeConfigure;
var RestH: LongInt;
begin
  RestH := WndH - LabH;
  LevelW := (WndW - Border * 3) div 2;
  LevelH := (RestH - Border * 4) div 3;
  FreqW := WndW - Border * 2;
  FreqH := (RestH - LevelH - Border * 4) div 2;
  SetMouseArea(msL, msActive, Border,		 Border, LevelW, LevelH, 15, 0, '', XC_left_side);
  SetMouseArea(msR, msActive, Border*2 + LevelW, Border, LevelW, LevelH, 15, 0, '', XC_right_side);
  SetMouseArea(msI, msActive, Border, Border*2 + LevelH,	 FreqW, FreqH, 14, 0, '', DefaultMouseCursor);
  SetMouseArea(msO, msActive, Border, Border*3 + LevelH + FreqH, FreqW, FreqH, 14, 0, '', DefaultMouseCursor);
end;


//       
procedure intDrawTuner(const mixrWnd: TWindow; var GC: TGC; const Colors: TColors; var dt);
var x: LongInt;
begin with tVolDesc(dt) do begin
  x := Round(vp^ / MaxVol * LevelW);
  XClearWindow(prDisplay, mixrWnd);
  XDrawLine(prDisplay, mixrWnd, GC, 0, LevelH div 2, LevelW, LevelH div 2);
  XSetForeGround(prDisplay, GC, Colors[12]);
  XFillRectangle(prDisplay, mixrWnd, GC, x - 1, 0, 3, LevelH);
end end;

procedure intDecVol(var dt);
begin with tVolDesc(dt) do begin
  if TestMKey(skLShift) then dec(vp^) else dec(vp^, 10);
  if vp^ < 0 then vp^ := MaxVol;
  mixrChangeVolume(0, 0);
end end;

procedure intIncVol(var dt);
begin with tVolDesc(dt) do begin
  if TestMKey(skLShift) then inc(vp^) else inc(vp^, 10);
  if vp^ > MaxVol then vp^ := 0;
  mixrChangeVolume(0, 0);
end end;

procedure intToVol(var dt);
begin with tVolDesc(dt) do begin
  vp^ := Round(MouseX / LevelW * MaxVol);
  if vp^ < 0 then vp^ := 0;
  if vp^ > MaxVol then vp^ := MaxVol;
  mixrChangeVolume(0, 0);
end end;


//        
procedure intDrawFreq(const mixrWnd: TWindow; var GC: TGC; const Colors: TColors; var dt);
begin with tFreqDesc(dt) do begin
  XClearWindow(prDisplay, mixrWnd);
  AlfCenter := True;
  AlfOutTextXYPS(mixrWnd, GC, 0, FreqW, FreqH div 2, ' ' + s + ': ' + ValToStrG(fp^) + ' /');
  AlfCenter := False;
  mixrNewUserFreq := True;
end end;

{$R-}
procedure intDecFreq(var dt);
begin with tFreqDesc(dt) do begin
  if TestMKey(skLShift) then
    if TestMKey(skLCtrl) then dec(fp^, 10000) else dec(fp^, 100)
  else
    if TestMKey(skLCtrl) then dec(fp^, 1000) else dec(fp^, 10);
  mixrNewUserFreq := True;
end end;

procedure intIncFreq(var dt);
begin with tFreqDesc(dt) do begin
  if TestMKey(skLShift) then
    if TestMKey(skLCtrl) then inc(fp^, 10000) else inc(fp^, 100)
  else
    if TestMKey(skLCtrl) then inc(fp^, 1000) else inc(fp^, 10);
  mixrNewUserFreq := True;
end end;

procedure intToFreq(var dt);
var l: LongInt;
begin with tFreqDesc(dt) do begin
  l := fp^;
  if AlfEnterValue(l, '  ' + s + ': ', MouseXroot, MouseYroot) then exit;
  if l > 65535 then l := 65535;
  if l < 0 then l := 0;
  fp^ := l;
end end;


procedure MyFullingTime(var Event: TXEvent; const BlockEn: Boolean);
var GC: TGC;
begin
  OldFullingTime(Event, BlockEn);
  case Event.EventType of

    Expose: with Event.XExpose do if Window = mixrWnd then begin
      GC := XCreateGC(prDisplay, mixrWnd, 0, Nil);
      AlfSetFont(GC, FontLabel);
      XSetForeGround(prDisplay, GC, Colors[7]);
      AlfOutTextXYPS(mixrWnd, GC, Border, Border, WndH - LabH, Remark);
      XFreeGC(prDisplay, GC);
    end;

    ConfigureNotify: with Event.XConfigure do if Window = mixrWnd then begin
      WndW := Width; WndH := Height;
      intSizeConfigure;
    end;

  end;
end;


procedure intLoadSetting;
var l: LongInt;
begin
  IniOpen('Mixer');
  IniGet('Mixer Command', MixerCMD);
  IniGet('Maximum Value', MaxVol);
  IniGet('Wnd X', WndX);
  IniGet('Wnd Y', WndY);
  IniGet('Wnd W', WndW);
  IniGet('Wnd H', WndH);
  IniGet('Font Name', FontLabel);
  if IniGet('Volume L', l) then VolL := l;
  if IniGet('Volume R', l) then VolR := l;
  IniClose;
end;


procedure intSaveSetting;
begin
  ExitProc := OldExitProc;
  IniOpen('Mixer');
  IniPut('Volume L', VolL);
  IniPut('Volume R', VolR);
  IniClose;
end;


begin
  intLoadSetting;
  if MixerCMD <> '' then begin
    NewRootWindow(WndX, WndY, WndW, WndH, 10, 40, '   OSS', '   OSS', mixrWnd);
    SetVGAPalette(mixrWnd, Colors);
    NewMouseArea(mixrWnd, msL);  NewMouseArea(mixrWnd, msR);
    NewMouseArea(mixrWnd, msI);  NewMouseArea(mixrWnd, msO);
    SetMouseRoutines(msL, @intDrawTuner, VolLdesc, @intDecVol, @intIncVol, @intToVol);
    SetMouseRoutines(msR, @intDrawTuner, VolRdesc, @intDecVol, @intIncVol, @intToVol);
    SetMouseRoutines(msI, @intDrawFreq, FreqIdesc, @intDecFreq, @intIncFreq, @intToFreq);
    SetMouseRoutines(msO, @intDrawFreq, FreqOdesc, @intDecFreq, @intIncFreq, @intToFreq);
    AlfFakeSetFont(FontLabel);
    LabH := ALfHeightPS(Remark);
    intSizeConfigure;
    OldFullingTime := FullingTime; FullingTime := @MyFullingTime;
    OldExitProc := ExitProc; ExitProc := @intSaveSetting;
  end;
end.
